@@ -103,6 +103,8 @@
 #' \item{level, incr}{As defined above.}
 #' \item{k}{The number of studies in the meta-analysis.}
 #' \item{optcut}{The optimal cutoff.}
+#' \item{lower.optcut, upper.optcut}{Corresponding lower and upper
+#'   confidence limits (for normal distribution).}
 #' \item{Sens.optcut}{The sensitivity at the optimal cutoff.}
 #' \item{lower.Sens.optcut, upper.Sens.optcut}{Corresponding lower and
 #'   upper confidence limits.}
@@ -551,6 +553,8 @@ diagmeta <- function(TP, FP, TN, FN, cutoff, studlab, data = NULL,
     }
     else
       optcut <- wmean
+    ##
+    var.optcut <- NA
   }
   else if (is.normal) {
     ## Cutoffs of two normals, weighted with lambda and 1 - lambda
@@ -624,6 +628,14 @@ diagmeta <- function(TP, FP, TN, FN, cutoff, studlab, data = NULL,
                                                                                2 * dalpha1 * dbeta0 * cov.alpha1.beta0
     }
   }
+  ##
+  ci.optcut <- ci(optcut, sqrt(var.optcut), level = level)
+  ##
+  if (log.cutoff) {
+    ci.optcut$TE    <- exp(ci.optcut$TE)
+    ci.optcut$lower <- exp(ci.optcut$lower)
+    ci.optcut$upper <- exp(ci.optcut$upper)
+  }
   
   
   ##
@@ -687,8 +699,9 @@ diagmeta <- function(TP, FP, TN, FN, cutoff, studlab, data = NULL,
   ## (10) List with results
   ##
   ##
-  res <- list(TP = TP, FP = FP, TN = TN, FN = FN,
-              cutoff = cutoff, studlab = studlab,
+  res <- list(studlab = studlab,
+              TP = TP, FP = FP, TN = TN, FN = FN,
+              cutoff = cutoff,
               ##
               Sens = 1 - (FN + incr) / (N1 + 2 * incr),
               Spec = (TN + incr) / (N0 + 2 * incr),
@@ -701,7 +714,9 @@ diagmeta <- function(TP, FP, TN, FN, cutoff, studlab, data = NULL,
               ##
               k = k,
               ##
-              optcut = if (log.cutoff) exp(optcut) else optcut,
+              optcut = ci.optcut$TE,
+              lower.optcut = ci.optcut$lower,
+              upper.optcut = ci.optcut$upper,
               ##
               Sens.optcut = Se,
               lower.Sens.optcut = lower.Se,
